#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
@文件名称: generate_local_settings.py
@功能描述: 自动生成 Django 本地配置文件（local_settings.py）
@参数说明: 无
@依赖环境: Python >= 3.6
@使用方法: python generate_local_settings.py
"""
import os
import re
import sys
import subprocess
from pathlib import Path


# 定义终端颜色代码
class TermColors:
    HEADER = '\033[95m'
    OKBLUE = '\033[94m'
    OKGREEN = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    ENDC = '\033[0m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'


def colored_print(text, color):
    """带颜色的打印函数"""
    print(f"{color}{text}{TermColors.ENDC}")


def install_requests():
    """安装 requests 库（若未安装），使用最新兼容版本"""
    try:
        import requests
    except ImportError:
        colored_print("未找到 requests 库，正在安装最新版...", TermColors.WARNING)
        try:
            # 首先检查pip是否可用
            try:
                subprocess.check_call(
                    [sys.executable, "-m", "pip", "--version"],
                    stderr=subprocess.DEVNULL,
                    stdout=subprocess.DEVNULL
                )
            except (subprocess.CalledProcessError, FileNotFoundError):
                colored_print("检测到pip不可用，尝试安装pip...", TermColors.WARNING)
                # 下载get-pip.py并安装pip
                try:
                    import os
                    import urllib.request
                    get_pip_url = "https://bootstrap.pypa.io/get-pip.py"
                    get_pip_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), "get-pip.py")
                    urllib.request.urlretrieve(get_pip_url, get_pip_path)
                    subprocess.check_call([sys.executable, get_pip_path])
                    os.remove(get_pip_path)
                    colored_print("pip安装成功", TermColors.OKGREEN)
                except Exception as e:
                    colored_print(f"pip安装失败: {e}", TermColors.FAIL)
                    colored_print("请手动安装pip后再运行此脚本", TermColors.FAIL)
                    sys.exit(1)

            # 使用pip安装requests
            subprocess.check_call(
                [sys.executable, "-m", "pip", "install", "requests"]
            )
            colored_print("requests 及依赖库安装成功。", TermColors.OKGREEN)
        except subprocess.CalledProcessError as e:
            colored_print(f"requests 库安装失败: {e}", TermColors.FAIL)
            colored_print("请手动安装requests库: pip install requests", TermColors.FAIL)
            sys.exit(1)


def get_server_ip():
    """获取服务器公网 IP（失败时返回 127.0.0.1）"""
    install_requests()
    import requests
    try:
        response = requests.get('https://ipinfo.io/ip', timeout=5)
        if response.status_code == 200:
            return response.text.strip()
    except requests.RequestException:
        pass
    return '127.0.0.1'


def get_env_from_bashrc():
    """从.bashrc文件中提取环境变量"""
    env_vars = {}
    bashrc_path = os.path.expanduser('~/.bashrc')

    if not os.path.exists(bashrc_path):
        return env_vars

    with open(bashrc_path, 'r') as f:
        content = f.read()

    # 匹配 export VAR="value" 或 export VAR='value'
    pattern = re.compile(r'export\s+([A-Z_]+)=["\']([^"\']+)["\']')
    matches = pattern.findall(content)

    for var, value in matches:
        env_vars[var] = value

    return env_vars


def get_project_name():
    """获取当前Django项目的名称"""
    try:
        # 尝试从环境变量获取
        project_name = os.getenv('DJANGO_PROJECT_NAME')
        if project_name:
            return project_name

        # 尝试从当前目录结构判断
        manage_py = Path.cwd() / 'manage.py'
        if manage_py.exists():
            # 标准Django项目结构: project_name/manage.py
            return Path.cwd().name

        # 查找settings.py的父目录名
        settings_path = find_settings_file()
        if settings_path.parent.name == settings_path.parent.parent.name:
            return settings_path.parent.name  # project_name/project_name/settings.py
        return settings_path.parent.name  # project_dir/settings.py

    except Exception:
        return 'myproject'  # 默认值


def find_settings_file():
    """查找settings.py文件路径"""
    search_paths = [Path.cwd()] + list(Path.cwd().parents)[:3]
    for base_dir in search_paths:
        # 标准结构：project_name/project_name/settings.py
        standard_path = base_dir / base_dir.name / "settings.py"
        if standard_path.exists():
            return standard_path
        # 单层结构：base_dir/settings.py
        if (base_dir / "settings.py").exists():
            return base_dir / "settings.py"
    raise FileNotFoundError("未找到 settings.py 文件")


def generate_local_settings():
    try:
        # 获取环境变量
        env_vars = get_env_from_bashrc()
        mysql_password = env_vars.get('MYSQL_ROOT_PASS', 'Chan1996!')
        redis_password = env_vars.get('REDIS_PASSWORD', 'chanjian8888')
        project_name = get_project_name()

        server_ip = get_server_ip()

        settings_dir = find_settings_file().parent
        base_dir = settings_dir.parent  # 项目根目录的上一级目录

        # 静态文件完整路径
        static_root_path = os.path.join(base_dir, f'{project_name}_staticfiles')

        content = f"""
DEBUG = False

ALLOWED_HOSTS = ['{server_ip}', 'localhost', '127.0.0.1']

DATABASES = {{
    'default': {{
        'ENGINE': 'django.db.backends.mysql',
        'NAME': '{project_name}',
        'USER': 'root',
        'PASSWORD': '{mysql_password}',
        'HOST': '127.0.0.1',
        'PORT': 3306,
        'OPTIONS': {{
            'init_command': "SET sql_mode='STRICT_TRANS_TABLES'",
        }}
    }}
}}

CACHES = {{
    "default": {{
        "BACKEND": "django_redis.cache.RedisCache",
        "LOCATION": "redis://127.0.0.1:6379/0",
        "OPTIONS": {{
            "CLIENT_CLASS": "django_redis.client.DefaultClient",
            "CONNECTION_POOL_KWARGS": {{"max_connections": 100}},
            "PASSWORD": "{redis_password}",
        }}
    }}
}}

import os
from pathlib import Path
# 获取当前项目的根目录
BASE_DIR = Path(__file__).resolve().parent.parent
# 动态生成 STATIC_ROOT 路径
STATIC_ROOT = os.path.join(BASE_DIR.parent, '{project_name}_staticfiles')
"""

        output_path = settings_dir / "local_settings.py"
        with open(output_path, 'w', encoding='utf-8') as f:
            f.write(content.strip())
        
        # 高亮显示成功信息
        success_msg = f"✔ 本地配置文件已生成: {output_path}"
        colored_print(success_msg, TermColors.OKGREEN)
        
        # 显示静态文件目录完整路径
        static_msg = f"✔ 静态文件将收集到: {static_root_path}"
        colored_print(static_msg, TermColors.OKBLUE)

    except Exception as e:
        error_msg = f"错误: {e}"
        colored_print(error_msg, TermColors.FAIL)
        sys.exit(1)


if __name__ == "__main__":
    generate_local_settings()